from object import Thing
import pygame
import data
from constants import *
from feather import Feather

class Player(Thing):
    def __init__(self):
        Thing.__init__(self)
        self.image = data.load_image('player.png')
        self.rect = self.image.get_rect()
        self.pos = GAME_RES[0]/2
        self.height = 10.0
        self.rect.move_ip(self.pos,GAME_RES[1]-self.rect.height-self.height) #move to the centre bottom of the screen
        self.score = 0
        self.feathers = {}
        self.goal = {}
        self.leftovertime = 0 #left over time from all achieved goals
        self.drops = 0 #number of times monkey has gotten you
        self.velocity = 0.0
        self.jumping = False
        self.falling = False
        self.jumpspeed = 0.0

    def move(self,dt):
        #move left or right
        start = self.rect.centerx
        self.pos += ((self.velocity * dt) / 1000)
        self.rect.centerx = self.pos
        if self.offscreen():
            self.pos = start
            self.rect.centerx = start #go back

        #handle jumping
        self.jumpspeed = 100.0+JUMP_FACTOR*(JUMP_HEIGHT-self.height)
        if self.jumping:
            self.height += ((self.jumpspeed * dt) / 1000)
            if self.height > JUMP_HEIGHT:
                self.height = JUMP_HEIGHT
                self.jumping = False
                self.falling = True
            self.rect.centery = GAME_RES[1]-self.rect.height/2 - self.height
        elif self.falling:
            self.height -= ((self.jumpspeed * dt) / 1000)
            if self.height < 10.0:
                self.height = 10.0
                self.falling = False
            self.rect.centery = GAME_RES[1]-self.rect.height/2 - self.height

    def add_feather(self,feather):
        ok = True
        color = feather.color
        if color in self.goal and self.goal[color] > 0:
            if color in self.feathers:
                if self.feathers[color] == self.goal[color]:
                    ok = False #too many
                    self.feathers = {}
                else:
                    self.feathers[color] += 1
            else:
                self.feathers[color] = 1
            if self._completed():
                self.score += 1
                self.goal = {}
                self.feathers = {}
        else:
            ok = False #wrong color
            self.feathers = {} #drop them all
        return ok

    def aimless(self):
        return not self.goal

    def _completed(self):
        result = True
        for i,j in self.goal.items():
            if (j != 0) and (i not in self.feathers or self.feathers[i] != j):
                result = False
                break
        return result

    def jump(self):
        self.jumping=True

    def left(self):
        self.velocity = -PLAYER_SPEED

    def right(self):
        self.velocity = PLAYER_SPEED

    def stop(self):
        self.velocity = 0

    def set_goal(self,goal):
        self.goal = goal

    def drop(self):
        self.feathers = {}
        self.drops += 1

    def collide_top(self,feather):
        return (feather.rect.centery < self.rect.top and pygame.sprite.collide_rect(self,feather))
